unit uRomanNumeralsTest;

interface

uses
  DelphiSpec.Attributes, DelphiSpec.StepDefinitions, DUnitX.TestFramework;

type
  TRomanNumeralConverter = class
  private
    FInput: Integer;
    function GetOutput: string;

  public
    property Input: Integer write FInput;
    property Output: string read GetOutput;
  end;



  [Feature('Roman numerals converter')]
  TRomanNumeralsTest = class(TStepDefinitions)
  private
    FConverter: TRomanNumeralConverter;
    FOutput: string;
  public
    [Given_('I have entered $Value into the converter')]
    procedure EnterNumber(Value: Integer);

    [When_('I ask for the Roman value')]
    procedure AskForValue;

    [Then_('it will give me $Value')]
    procedure ValidateResult(Value: string);

    procedure SetUp; override;
    procedure TearDown; override;
  end;

implementation

uses
  DelphiSpec.Core;


{ TRomanNumeralsTest }

procedure TRomanNumeralsTest.AskForValue;
begin
  FOutput := FConverter.Output;
end;

procedure TRomanNumeralsTest.EnterNumber(Value: Integer);
begin
  FConverter.Input := Value;
end;

procedure TRomanNumeralsTest.SetUp;
begin
  inherited;

  FConverter := TRomanNumeralConverter.Create;
end;

procedure TRomanNumeralsTest.TearDown;
begin
  inherited;

  FConverter.Free;
end;

procedure TRomanNumeralsTest.ValidateResult(Value: string);
begin
  Assert.AreEqual(Value, FOutput);
end;

{ TRomanNumeralConverter }

function TRomanNumeralConverter.GetOutput: string;
begin
  case FInput of
    1: Result := 'I';
    11: Result := 'XI';
    1979: Result := 'MCMLXXIX';
    else Result := '';
  end;
end;

initialization
  RegisterStepDefinitionsClass(TRomanNumeralsTest);

end.
